#######################################################################
#
# chkconfig component
#
# NCM chkconfig component
#
# Teemu Sidoroff <Teemu.Sidoroff@cern.ch>
# Version 1.0.5 : 20/08/04 12:55
#
# For license conditions see http://www.eu-datagrid.org/license.html
#
#######################################################################

package NCM::Component::chkconfig;

#
# a few standard statements, mandatory for all components
#

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;

use NCM::Check;
use LC::Process qw(run);

##########################################################################
sub Configure {
##########################################################################
  my ($self,$config)=@_;
  my $valPath = '/software/components/chkconfig/service';

  unless($config->elementExists($valPath)) {
      $self->error("cannot get $valPath");
      return;
  }
  my $srvcs = $config->getElement($valPath);

  my @cmdlist;
  my $chkconfigcmd = "/sbin/chkconfig";
  my $servicecmd   = "/sbin/service";

  while($srvcs->hasNextElement()) {
      
      my $startstop;
      my $cs = $srvcs->getNextElement(); #current service
      my $service = $cs->getName();

      #get startstop value if it exists
      my $servpath = $cs->getPath()->toString();

      if($config->elementExists("$servpath/startstop")) {
	  my $ssel = $config->getElement("$servpath/startstop");
	  $startstop = $ssel->getValue();
      }

      #override the service name to use value of 'name' if it is set
      if($config->elementExists("$servpath/name")) {
	  my $nameel = $config->getElement("$servpath/name");
	  $service = $nameel->getValue();
      }


      while($cs->hasNextElement()) {
	  
	  my $opt = $cs->getNextElement();
	  my $optname = lc($opt->getName());
	  my $optval =  lc($opt->getValue());
	  chomp $optval;

	  #6 kinds of entries: on,off,reset,add,del and startstop.
	  if($optname eq 'add' and $optval eq 'true') {
	      push @cmdlist, "$chkconfigcmd --add $service";

	      if($startstop and $startstop eq 'true') {
		  push @cmdlist, "$servicecmd $service start";
	      }
	
	  } elsif ($optname eq 'del' and $optval eq 'true') {
	      push @cmdlist, "$chkconfigcmd --del $service";

	      if($startstop and $startstop eq 'true') {
		  push @cmdlist, "$servicecmd $service stop";
	      }

	  } elsif ($optname eq 'on') { 
	      if(validrunlevels($optval)) {
		  if($optval) {
		      push @cmdlist, "$chkconfigcmd --level $optval $service on";

		  } else {
		      push @cmdlist, "$chkconfigcmd $service on";
		  } 		  

		  if($startstop and $startstop eq 'true'  
		     and ((!$optval) or ($optval =~ /3/))) {
		      push @cmdlist, "$servicecmd $service start";
		  }

	      } else { 
		  $self->warn("invalid runlevel string $optval defined for ".
			      "option \'$optname\' in service $service");
	      }
	     
	  } elsif ($optname eq 'off') { 
	      if(validrunlevels($optval)) {
		  if($optval) {
		      push @cmdlist, "$chkconfigcmd --level $optval $service off";
		  } else {
		      push @cmdlist, "$chkconfigcmd $service off";
		  } 

		  if($startstop and $startstop eq 'true'
		     and ((!$optval) or ($optval =~ /3/))) {
		      push @cmdlist, "$servicecmd $service stop";
		  }
	      } else {
 		  $self->warn("invalid runlevel string $optval defined for ".
			      "option \'$optname\' in service $service");
	      }

	  } elsif ($optname eq 'reset') {
	      if(validrunlevels($optval)) {
		  if($optval) {
		      push @cmdlist, "$chkconfigcmd --level $optval $service reset";
		  } else {
		      push @cmdlist, "$chkconfigcmd $service reset";
		  } 
	      } else {
		  $self->warn("invalid runlevel string $optval defined for ".
			      "option $optname in service $service");
	      }

	  } elsif ($optname eq 'startstop' or $optname eq 'add' or 
		   $optname eq 'del' or $optname eq 'name') {
	      # do nothing
	  } else {
	      $self->error("undefined option name $optname in service $service");
	      return;
	  }
      } #while
  } #while
  
  #perform the commands 
  for my $cmd (@cmdlist) {
      #info 
      $self->info("executing command: $cmd");
      
      unless($NoAction) {
	  unless(run("$cmd")) {
	      $self->warn("cannot execute $cmd");
	  } 
      }
  }
  return;
}

##########################################################################
sub Unconfigure {
##########################################################################
}

##########################################################################
sub validrunlevels(\$) {
##########################################################################
    my $str = shift;
    chomp($str);
    
    return 1 unless ($str);

    if($str =~ /^[0-7]+$/) {
	return 1;
    }

    return 0;
}

1; #required for Perl modules
